<?
/**
 * SteamInfo: Steam-Server Informationen abfragen.
 * Mit SteamInfo lassen sich simpel Informationen
 * ber einen Steam-Gameserver erfragen. Dabei werden
 * Fehler per Standart Exceptions geschmissen.
 * 
 * Geplant ist die Klasse zu verbessern und auszubauen,
 * evtl automatisches sortieren etc.
 * Evtl. deswegen weil ursprnglich nur eine Klasse
 * zum einlesen der Informationen in sinnvolle Arrays
 * zu Weiterverarbeitung gedacht war.
 * 
 * Bei Fragen oder Anregungen: jeanpascal@hotmail.de
 * 
 *CHANGELOG 
 * 27. Mai 2006 | 3:09 | v1.0.0.1
 * 		Erstes Release!
 * 		Todo:
 * 		Ungenau, Codeverbesserungen..
 * 
 *
 * @version 1.0.0.1
 * @author  Jean-Pascal Hvel
 * @copyright Jean-Pascal Hvel 
 * @package SteamInfo-v1.0.0.1
 * @example example.php Beispiel
 * 
 * 
 * 
 */
class SteamInfo {
	
	/**
	 * Socket-Connection zum 
	 * Steam Server.
	 *
	 * @var resource
	 */
	private $_rSocket;
	
	/**
	 * Array mit Commands 
	 * die zum Server gesendet werden. 
	 *
	 * @var array
	 */
	private $_aCommand;
	
	/**
	 * Challange Nummer die
	 * vom Server empfangen wurde.
	 *
	 * @var string
	 */
	private $_sChNumber;
		
	/**
	 * Hostname des Steamservers
	 *
	 * @var string
	 */
	private $_sHost;
	
	/**
	 * Port des Steamservers
	 *
	 * @var int
	 */
	private $_iPort;
	
	/**
	 * Timeout der Connection
	 *
	 * @var int
	 */
	private $_iTimeout;
	
	/**
	 * Konnektiert den Steamserver.
	 * Gibt false zurck wenn die Verbindung fehlschlug.
	 *
	 * @return bool
	 */
	private function _connect(){
		if(!is_resource($this->_rSocket)){
			$null = null;
			if(!is_resource($this->_rSocket = @fsockopen("udp://" . $this->_sHost, $this->_iPort, $null, $null, $this->_iTimeout))){
				unset($null);
				return false;
			}
			else {
				unset($null); 
				return true;
			}
		}
		else return true;
	}
	
	/**
	 * Schliesst die Connection zum Steam Server
	 * 
	 * @return void
	 */
	private function _disconnect(){
		if(is_resource($this->_rSocket)) @fclose($this->_rSocket);
	}
	
	/**
	 * Sendet ein Kommando an den Steamserver
	 * und gibt deren Antwort zurck.
	 *
	 * @param  string $cmd
	 * @param  string $add
	 * @return string
	 */
	private function _sendCommand($cmd, $add = false){
		$send = $this->_aCommand[$cmd] . ($add !== false ? $add : "");
		@fwrite($this->_rSocket, $send);
		$return = "";
		$loop   = true;
		while($loop){
			$return .= @fread($this->_rSocket, 1);
			$check  = socket_get_status($this->_rSocket);
			$loop 	= ($check['unread_bytes'] !== 0 ? true : false);
		}
		unset($loop, $send, $cmd, $check);
		return $return;
	}
	
	/**
	 * Holt die Challange Nummer vom Steam 
	 * Server, die fr verschiedene Befehle bentigt wird.
	 *
	 * @return void
	 */
	private function _getChallengeNumber(){
		$n = $this->_sendCommand('getNumber');
		$this->_sChNumber = substr($n, 5, 4);
	}
	
	/**
	 * Erzeugt einen String aus den Zeichen
	 * beginnend bei $start. Endet bei ASCII Code 0.
	 * Als Rckgabe wird ein Array mit dem erzeugtem String
	 * sowie der neuen Position zurckgegeben.
	 * 
	 * @param  int    $start
	 * @param  string $input
	 * @return array
	 */
	private function _generateString($start, $input){
		for($i = $start; $i < strlen($input) && $input{$i} !== chr(0); $i++);
		return array(
			'string' => substr($input, $start, $i-$start),
			'pos'	 => $i+1);
	}
	
	/**
	 * Erzeugt ein neues SteamInfo Objekt.
	 * Bei fehlender oder falscher Parameter
	 * Angabe wird eine Exception geworfen.
	 * Standartmig ist Port 27015 und ein
	 * Timeout von 10 eingestellt.
	 * 
	 * @throws Exception
	 * @param string $host
	 * @param string $port
	 * @param int $timeout
	 * @return SteamInfo
	 */
	public function SteamInfo($host, $port = 27015, $timeout = 10){
		if(!is_string($host) || !is_int($port) || !is_int($timeout)){
			throw new Exception("Invalid parameters, need \$host String, \$port Int, \$timeout int (optional)");
		}
		else {
			$this->_sHost  	  = $host;
			$this->_iPort 	  = $port;
			$this->_iTimeout  = $timeout;
			$this->_rSocket  = null;
			$this->_aCommand = array(
					'getNumber' => "\xFF\xFF\xFF\xFF\x57",
					'getInfo'   => "\xFF\xFF\xFF\xFF\x54\x53\x6F\x75\x72\x63\x65\x20\x45\x6E\x67\x69\x6E\x65\x20\x51\x75\x65\x72\x79\x00",
					'getList'	=> "\xFF\xFF\xFF\xFF\x55",
					'getRules'	=> "\xFF\xFF\xFF\xFF\x56");
			unset($host, $port, $timeout);
		}
	}
	
	/**
	 * Liefert ein Array der Spieler auf
	 * dem Steam Server.
	 * Struktur
	 * $array[0]['name'] 	Name des Spielers
	 * $array[0]['frags']	Frags des Spielers
	 * $array[0]['time']	Timestamp der Spielzeit
	 *
	 * @throws Exception
	 * @return array
	 */
	public function getPlayerList(){
		if($this->_connect()){
			if(!$this->_sChNumber) $this->_getChallengeNumber();
			$reply = $this->_sendCommand('getList', $this->_sChNumber);
			if(substr($reply, 0, 4) !== "\xFF\xFF\xFF\xFF") throw new Exception("No answer from server $this->_sHost:$this->_iPort (getPlayerList())");
			$reply  = substr($reply, 4);
			$return = array();
			$count  = ord($reply{1});
			$i 		= 2;
			for($n = 0; $n < $count; $n++){
				$i += 1;
				$return[$n] = array();
				$return[$n]['name'] = $this->_generateString($i, $reply);
				$i = $return[$n]['name']['pos'];
				$return[$n]['name']  = $return[$n]['name']['string'];
				$return[$n]['frags'] = unpack("L", substr($reply, $i, $i+4));
				$return[$n]['frags'] = $return[$n]['frags'][1];
				$i += 4;
				$return[$n]['time']  = unpack("f", substr($reply, $i, $i+4));
				$return[$n]['time']  = mktime(0, 0, $return[$n]['time'][1]);
				$i += 4;
			}
			unset($reply, $i, $n);
			return $return;
		}
		else throw new Exception("Couldnt connect to $this->_sHost:$this->_iPort.(Timeout = $this->_iTimeout)");
	}
	
	/**
	 * Liefert ein Array mit den Servereinstellungen
	 * Struktur
	 * $array['cvar'] = 'value'
	 *
	 * @throws Exception
	 * @return array
	 */
	public function getServerRules(){
		if($this->_connect()){	
			if(!$this->_sChNumber) $this->_getChallengeNumber();
			$reply = $this->_sendCommand('getRules', $this->_sChNumber);
			if(substr($reply, 0, 4) !== "\xFF\xFF\xFF\xFF") throw new Exception("No answer from server $this->_sHost:$this->_iPort (getServerRules())");
			$reply  = substr($reply, 4);
			$reply  = explode("\x00", $reply);
			$return = array();
			for($i = 2; $i < count($reply); $i += 2){		
				$return[$reply[$i-1]] = $reply[$i];
			}
			unset($reply, $i);
			return $return;
		}
		else throw new Exception("Couldnt connect to $this->_sHost:$this->_iPort.(Timeout = $this->_iTimeout)");	
	}
	
	/**
	 * Liefert ein Array aus Serverinformationen
	 * Enthalten sind die Keys:
	 * hostname:	Name des Servers
	 * map:			Momentane Map
	 * player:		Anzahl Spieler
	 * maxplayer:	Anzahl maximaler Spieler
	 * moddir:		Verzeichnis des Mods
	 * modname:		Name des Mods
	 * appid:		Steams "AppID" des Mods
	 * os:			Operating System des Servers
	 * passwd:		Passwort true/false
	 * secure:		VAC true/false
	 *
	 * @throws Exception
	 * @return array
	 */
	public function getServerInfo(){
		if($this->_connect()){	
			$reply  = $this->_sendCommand('getInfo');	
			$this->_disconnect();
			if(substr($reply, 0, 4) !== "\xFF\xFF\xFF\xFF") throw new Exception("No answer from server $this->_sHost:$this->_iPort (getServerInfo())");	
			$reply  = substr($reply, 4); 	
			$return = array();
			$return['hostname']  = $this->_generateString(2, $reply);
			$i = $return['hostname']['pos'];
			$return['hostname']  = $return['hostname']['string'];
			$return['map'] 		 = $this->_generateString($i, $reply);
			$i = $return['map']['pos'];
			$return['map']  	 = $return['map']['string'];	
			$return['moddir'] 	 = $this->_generateString($i, $reply);
			$i = $return['moddir']['pos'];
			$return['moddir']  	 = $return['moddir']['string'];		
			$return['modname'] 	 = $this->_generateString($i, $reply);
			$i = $return['modname']['pos'];
			$return['modname']   = $return['modname']['string'];
			$return['appid']	 = ord($reply{$i++}.$reply{$i++});
			$return['player']	 = ord($reply{$i++});
			$return['maxplayer'] = ord($reply{$i++});
			$return['bots']		 = ord($reply{$i++});	
			$return['dedicated'] = $reply{$i++} === 'l' ? false : true;
			$return['os']		 = $reply{$i++} === 'l' ? 'linux' : 'windows';
			$return['passwd']	 = ord($reply{$i++});
			$return['secure']	 = ord($reply{$i++});			
			unset($i, $reply);			
			return $return;
		}
		else throw new Exception("Couldnt connect to $this->_sHost:$this->_iPort.(Timeout = $this->_iTimeout)");
	}
}
?>